'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var preact = require('preact');
var assign = _interopDefault(require('object-assign'));

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Higher order component used to add validation capabilities to a form input.
 * @param {Component} WrappedComponent - preact component to add validation
 * @prop {String|Function} validate - string (not-emtpy, email, checked)
 * or custom function used to validate input. function is called with the value
 * of the field.
 * @prop {Function} onValidityChange - function called with the validity
 * status and message when the valdidity of the input changes.
 * @prop {Boolean} forceValidate - any time this value is set to true, the
 * field will validate, even if its content has not changed.
 * @return {Component} the same component you passed in, with validation
 */
function withValidation(WrappedComponent) {
  return function (_Component) {
    _inherits(ValidatedInput, _Component);

    function ValidatedInput(props) {
      _classCallCheck(this, ValidatedInput);

      var _this = _possibleConstructorReturn(this, (ValidatedInput.__proto__ || Object.getPrototypeOf(ValidatedInput)).call(this, props));

      _this.state = { status: null, message: null, value: props.value
        // if there's no validation, auto-success
      };if (!props.validate) _this.state.status = 'success';
      return _this;
    }

    _createClass(ValidatedInput, [{
      key: 'componentDidUpdate',
      value: function componentDidUpdate(prevProps, prevState) {
        // when validity status changes, reflect to parent if applicable
        if (this.state.status !== prevState.status) {
          this.props.onValidityChange && this.props.onValidityChange(this.state.status, this.state.message);
        }

        // if forceValidate is changed to a truthy value, and the field has
        // not been validated, run validate
        if (this.props.forceValidate && this.props.forceValidate !== prevProps.forceValidate && this.state.status === null) {
          this.validate(this.state.value);
        }
      }
    }, {
      key: 'validate',
      value: function validate(value) {
        var _this2 = this;

        if (!value) value = '';
        var opt = this.props.validate;
        this.setState({ value: value });

        // if it's a custom function, run it and use its return values
        // can return a promise if its async
        if (typeof opt === 'function') {
          var fn = opt(value);
          this.setState({ status: 'loading' });
          if (fn.then) {
            fn.then(function (res) {
              _this2.setState({
                status: res.status,
                message: res.message || null
              });
            });
          } else {
            this.setState({
              status: fn.status,
              message: fn.message || null
            });
          }
        }

        // if its a string preset, run the preset
        if (typeof opt === 'string') {
          if (opt === 'email') {
            if (value.match(/.+@.+\..+/)) {
              this.setState({ status: 'success', message: null });
            } else {
              this.setState({
                status: 'error',
                message: 'Please enter a valid email address'
              });
            }
          }

          if (opt === 'not-empty') {
            if (value.match(/.+/)) {
              this.setState({ status: 'success', message: null });
            } else {
              this.setState({
                status: 'error',
                message: 'This field cannot be empty'
              });
            }
          }

          if (opt === 'checked') {
            if (value) {
              this.setState({ status: 'success', message: null });
            } else {
              this.setState({
                status: 'error',
                message: 'You must accept in order to proceed.'
              });
            }
          }
        }

        // pass the value through to the parent's onValueChange if present
        if (this.props.onValueChange) {
          this.props.onValueChange(value);
        }
      }
    }, {
      key: 'render',
      value: function render() {
        // don't pass props meant only for the validation wrapper
        var dedupeProps = assign({}, this.props);
        delete dedupeProps.onValidityChange;
        delete dedupeProps.validate;

        // we hijack the valueChange function to run our validation, but call
        // the parent's function with the value as well
        delete dedupeProps.onValueChange;

        return preact.h(
          'div',
          { 'class': 'g-validation-container ' + this.state.status },
          preact.h(WrappedComponent, _extends({
            onValueChange: this.validate.bind(this)
          }, dedupeProps)),
          this.props.validate && preact.h('div', {
            'class': 'message',
            dangerouslySetInnerHTML: { __html: this.state.message }
          })
        );
      }
    }]);

    return ValidatedInput;
  }(preact.Component);
}

module.exports = withValidation;
